/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.database.Cursor;
import android.util.Log;

import net.micode.notes.tool.GTaskStringUtils;

import org.json.JSONException;
import org.json.JSONObject;


public class MetaData extends Task {
    // 定义一个常量，用于日志输出
    private final static String TAG = MetaData.class.getSimpleName();

    // 定义一个私有变量，用于存储相关任务ID
    private String mRelatedGid = null;

    // 设置元数据
    public void setMeta(String gid, JSONObject metaInfo) {
        try {
            // 将相关任务ID添加到元数据中
            metaInfo.put(GTaskStringUtils.META_HEAD_GTASK_ID, gid);
        } catch (JSONException e) {
            // 日志输出错误信息
            Log.e(TAG, "failed to put related gid");
        }
        // 设置备注
        setNotes(metaInfo.toString());
        // 设置名称
        setName(GTaskStringUtils.META_NOTE_NAME);
    }

    // 获取相关任务ID
    public String getRelatedGid() {
        return mRelatedGid;
    }

    // 判断是否需要保存
    @Override
    public boolean isWorthSaving() {
        return getNotes() != null;
    }

    // 根据远程JSON设置内容
    @Override
    public void setContentByRemoteJSON(JSONObject js) {
        super.setContentByRemoteJSON(js);
        if (getNotes() != null) {
            try {
                // 将备注转换为JSON对象
                JSONObject metaInfo = new JSONObject(getNotes().trim());
                // 获取相关任务ID
                mRelatedGid = metaInfo.getString(GTaskStringUtils.META_HEAD_GTASK_ID);
            } catch (JSONException e) {
                // 日志输出警告信息
                Log.w(TAG, "failed to get related gid");
                // 将相关任务ID设置为null
                mRelatedGid = null;
            }
        }
    }

    // 根据本地JSON设置内容
    @Override
    public void setContentByLocalJSON(JSONObject js) {
        // this function should not be called
        throw new IllegalAccessError("MetaData:setContentByLocalJSON should not be called");
    }

    // 从内容中获取本地JSON
    @Override
    public JSONObject getLocalJSONFromContent() {
        throw new IllegalAccessError("MetaData:getLocalJSONFromContent should not be called");
    }

    // 获取同步操作
    @Override
    public int getSyncAction(Cursor c) {
        throw new IllegalAccessError("MetaData:getSyncAction should not be called");
    }

}
