/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.ContentUris;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.NoteColumns;


public class AlarmInitReceiver extends BroadcastReceiver {

    // 定义查询所需的列
    private static final String [] PROJECTION = new String [] {
        NoteColumns.ID,            // 笔记的ID
        NoteColumns.ALERTED_DATE   // 笔记的提醒日期
    };



    // 定义列的索引
    private static final int COLUMN_ID                = 0;    // ID列的索引
    private static final int COLUMN_ALERTED_DATE      = 1;    // 提醒日期列的索引

    @Override
    public void onReceive(Context context, Intent intent) {
        // 获取当前时间
        long currentDate = System.currentTimeMillis();
        // 查询数据库中提醒日期大于当前时间且类型为笔记的记录
        Cursor c = context.getContentResolver().query(Notes.CONTENT_NOTE_URI,
                PROJECTION,
                NoteColumns.ALERTED_DATE + ">? AND " + NoteColumns.TYPE + "=" + Notes.TYPE_NOTE,
                new String[] { String.valueOf(currentDate) }, // 查询条件参数
                null); // 排序参数

        // 如果查询结果不为空
        if (c != null) {
            // 如果有记录
            if (c.moveToFirst()) {
                // 遍历所有记录
                do {
                    // 获取提醒日期
                    long alertDate = c.getLong(COLUMN_ALERTED_DATE);
                    // 创建一个新的Intent，用于启动AlarmReceiver
                    Intent sender = new Intent(context, AlarmReceiver.class);
                    // 设置Intent的数据URI，包含笔记的ID
                    sender.setData(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, c.getLong(COLUMN_ID)));
                    // 创建PendingIntent，用于延迟执行Intent
                    PendingIntent pendingIntent = PendingIntent.getBroadcast(context, 0, sender, 0);
                    // 获取AlarmManager服务
                    AlarmManager alermManager = (AlarmManager) context
                            .getSystemService(Context.ALARM_SERVICE);
                    // 设置闹钟，在指定时间唤醒设备并执行Intent
                    alermManager.set(AlarmManager.RTC_WAKEUP, alertDate, pendingIntent);
                } while (c.moveToNext()); // 移动到下一条记录
            }
            // 关闭Cursor
            c.close();
        }
    }
}
