/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.content.Context;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.PopupMenu;
import android.widget.PopupMenu.OnMenuItemClickListener;

import net.micode.notes.R;

public class DropdownMenu {
    // 声明一个Button对象，用于显示下拉菜单的按钮
    private Button mButton;
    // 声明一个PopupMenu对象，用于创建和管理下拉菜单
    private PopupMenu mPopupMenu;
    // 声明一个Menu对象，用于获取菜单项
    private Menu mMenu;

    // 构造方法，初始化DropdownMenu对象
    public DropdownMenu(Context context, Button button, int menuId) {
        // 将传入的Button对象赋值给mButton
        mButton = button;
        // 设置按钮的背景资源为下拉图标
        mButton.setBackgroundResource(R.drawable.dropdown_icon);
        // 创建一个PopupMenu对象，传入上下文和按钮对象
        mPopupMenu = new PopupMenu(context, mButton);
        // 获取PopupMenu的Menu对象
        mMenu = mPopupMenu.getMenu();
        // 使用PopupMenu的MenuInflater对象加载菜单布局文件
        mPopupMenu.getMenuInflater().inflate(menuId, mMenu);
        // 设置按钮的点击监听器
        mButton.setOnClickListener(new OnClickListener() {
            // 点击按钮时显示PopupMenu
            public void onClick(View v) {
                mPopupMenu.show();
            }
        });
    }

    // 设置下拉菜单项的点击监听器
    // 定义一个方法用于设置下拉菜单项的点击监听器
    public void setOnDropdownMenuItemClickListener(OnMenuItemClickListener listener) {
        // 如果PopupMenu对象不为空，则设置菜单项点击监听器
        if (mPopupMenu != null) {
            mPopupMenu.setOnMenuItemClickListener(listener);
        }
    }

    // 根据菜单项的ID查找菜单项
    public MenuItem findItem(int id) {

        // 调用mMenu对象的findItem方法，传入菜单项的id
        // mMenu是一个菜单对象，用于管理菜单项
        // findItem方法用于在菜单中查找指定id的菜单项
        // id是菜单项的唯一标识符，通常是一个整型常量
        // 返回找到的菜单项对象，如果未找到则返回null
        return mMenu.findItem(id);
    }

    // 设置按钮的标题
    public void setTitle(CharSequence title) {
        // 将传入的标题设置给按钮
        // 参数说明：
        // CharSequence title - 要设置的标题文本
        // 功能说明：
        // 该方法用于将传入的标题文本设置给成员变量mButton，即按钮的文本内容
        mButton.setText(title);
    }
}
