/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.content.Context;
import android.database.Cursor;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CursorAdapter;
import android.widget.LinearLayout;
import android.widget.TextView;

import net.micode.notes.R;
import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.NoteColumns;


public class FoldersListAdapter extends CursorAdapter {
    // 定义查询的列
    public static final String [] PROJECTION = {
        NoteColumns.ID, // 笔记ID
        NoteColumns.SNIPPET // 笔记摘要
    };



    // 定义列的索引
    public static final int ID_COLUMN   = 0; // ID列的索引
    public static final int NAME_COLUMN = 1; // 名称列的索引

    // 构造函数
    public FoldersListAdapter(Context context, Cursor c) {
        super(context, c);
        // TODO Auto-generated constructor stub
    }

    // 创建新的视图
    @Override
    public View newView(Context context, Cursor cursor, ViewGroup parent) {
        return new FolderListItem(context); // 返回一个新的FolderListItem视图
    }

    // 绑定数据到视图
    @Override
    public void bindView(View view, Context context, Cursor cursor) {
        if (view instanceof FolderListItem) { // 如果视图是FolderListItem类型
            // 获取文件夹名称，如果是根文件夹则显示特定的字符串
            String folderName = (cursor.getLong(ID_COLUMN) == Notes.ID_ROOT_FOLDER) ? context
                    .getString(R.string.menu_move_parent_folder) : cursor.getString(NAME_COLUMN);
            ((FolderListItem) view).bind(folderName); // 绑定文件夹名称到视图
        }
    }

    // 获取指定位置的文件夹名称
    public String getFolderName(Context context, int position) {
        Cursor cursor = (Cursor) getItem(position); // 获取指定位置的Cursor
        // 如果是根文件夹则显示特定的字符串
        return (cursor.getLong(ID_COLUMN) == Notes.ID_ROOT_FOLDER) ? context
                .getString(R.string.menu_move_parent_folder) : cursor.getString(NAME_COLUMN);
    }

    // 内部类FolderListItem，继承自LinearLayout
    private class FolderListItem extends LinearLayout {
        private TextView mName; // 文本视图，用于显示文件夹名称

        // 构造函数
        public FolderListItem(Context context) {
            super(context);
            inflate(context, R.layout.folder_list_item, this); // 加载布局文件
            mName = (TextView) findViewById(R.id.tv_folder_name); // 获取布局中的文本视图
        }

        // 绑定文件夹名称到文本视图
        public void bind(String name) {
            mName.setText(name); // 设置文本视图的文本
        }
    }

}
